@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 See PEP 440 (https://www.python.org/dev/peps/pep-0440/)
 
@@ -11,12 +10,11 @@ Release logic:
  4. git add djangocms_text_ckeditor/__init__.py CHANGELOG.rst setup.py
  5. git commit -m 'Bump to {new version}'
  6. git push
- 7. Assure that all tests pass on https://travis-ci.org/github/divio/djangocms-text-ckeditor.
- 8. git tag {new_version_number}
- 9. git push --tags
-10. python setup.py sdist
-11. twine upload dist/djangocms-text-ckeditor-{new_version_number}.tar.gz
+ 7. Assure that all tests pass on https://github.com/django-cms/djangocms-text-ckeditor/actions
+ 8. Create a new release on https://github.com/django-cms/djangocms-text-ckeditor/releases/new
+ 9. Publish the release when ready
+10. Github actions will publish the new package to pypi
 """
-__version__ = '3.10.0'
+__version__ = '5.0.0'
 
 default_app_config = 'djangocms_text_ckeditor.apps.TextCkeditorConfig'
