@@ -7,15 +7,12 @@ from django.core import signing
 from django.core.exceptions import PermissionDenied, ValidationError
 from django.db import transaction
 from django.forms.fields import CharField
-from django.http import (
-    Http404, HttpResponse, HttpResponseBadRequest, HttpResponseForbidden,
-    HttpResponseRedirect,
-)
+from django.http import Http404, HttpResponse, HttpResponseBadRequest, HttpResponseForbidden, HttpResponseRedirect
 from django.shortcuts import get_object_or_404
 from django.template import RequestContext
 from django.urls import re_path, reverse
 from django.utils.decorators import method_decorator
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 from django.utils.translation import gettext
 from django.views.decorators.clickjacking import xframe_options_sameorigin
 from django.views.decorators.http import require_POST
@@ -28,15 +25,11 @@ from cms.utils.placeholder import get_toolbar_plugin_struct
 from cms.utils.urlutils import admin_reverse
 
 from . import settings
-from .forms import (
-    ActionTokenValidationForm, DeleteOnCancelForm, RenderPluginForm, TextForm,
-)
+from .forms import ActionTokenValidationForm, DeleteOnCancelForm, RenderPluginForm, TextForm
 from .models import Text
 from .utils import (
-    OBJ_ADMIN_WITH_CONTENT_RE_PATTERN, _plugin_tags_to_html,
-    plugin_tags_to_admin_html, plugin_tags_to_id_list,
-    plugin_tags_to_user_html, plugin_to_tag, random_comment_exempt,
-    replace_plugin_tags,
+    OBJ_ADMIN_WITH_CONTENT_RE_PATTERN, _plugin_tags_to_html, plugin_tags_to_admin_html, plugin_tags_to_id_list,
+    plugin_tags_to_user_html, plugin_to_tag, random_comment_exempt, replace_plugin_tags,
 )
 from .widgets import TextEditorWidget
 
@@ -193,11 +186,11 @@ class TextPlugin(CMSPluginBase):
         _has_do_post_copy = True
 
     @classmethod
-    def do_post_copy(self, instance, source_map):
+    def do_post_copy(cls, instance, source_map):
         ids = plugin_tags_to_id_list(instance.body)
         ids_map = {pk: source_map[pk].pk for pk in ids if pk in source_map}
         new_text = replace_plugin_tags(instance.body, ids_map)
-        self.model.objects.filter(pk=instance.pk).update(body=new_text)
+        cls.model.objects.filter(pk=instance.pk).update(body=new_text)
 
     @staticmethod
     def get_translation_export_content(field, plugin_data):
@@ -329,7 +322,7 @@ class TextPlugin(CMSPluginBase):
             # The instance is a record that points to the Text plugin
             # but is not a real text plugin instance.
             return super().add_view(
-                request, form_url, extra_context
+                request, form_url, extra_context,
             )
 
         if not self.has_add_permission(request):
@@ -339,7 +332,7 @@ class TextPlugin(CMSPluginBase):
             # on GET request to the /add/ endpoint and so we bypass
             # django's add_view, thus bypassing permission check.
             message = gettext('You do not have permission to add a plugin.')
-            return HttpResponseForbidden(force_text(message))
+            return HttpResponseForbidden(force_str(message))
 
         try:
             # CMS 3.3 compatibility
@@ -358,7 +351,7 @@ class TextPlugin(CMSPluginBase):
 
         except PermissionDenied:
             message = gettext('You do not have permission to add a plugin.')
-            return HttpResponseForbidden(force_text(message))
+            return HttpResponseForbidden(force_str(message))
         except ValidationError as error:
             return HttpResponseBadRequest(error.message)
 
@@ -395,7 +388,7 @@ class TextPlugin(CMSPluginBase):
 
     def get_admin_url_name(self, name):
         plugin_type = self.__class__.__name__.lower()
-        url_name = '%s_%s_%s' % (self.model._meta.app_label, plugin_type, name)
+        url_name = f'{self.model._meta.app_label}_{plugin_type}_{name}'
         return url_name
 
     def _get_text_plugin_from_request(self, request, data):
@@ -411,7 +404,7 @@ class TextPlugin(CMSPluginBase):
             if text_plugin_id:
                 return self._get_plugin_or_404(text_plugin_id)
         message = gettext('Unable to process your request. Invalid token.')
-        raise ValidationError(message=force_text(message))
+        raise ValidationError(message=force_str(message))
 
     @random_comment_exempt
     @xframe_options_sameorigin
@@ -513,7 +506,7 @@ class TextPlugin(CMSPluginBase):
                 context,
             ),
             'placeholder': placeholder,
-            'object': instance
+            'object': instance,
         })
         return context
 
@@ -537,9 +530,10 @@ class TextPlugin(CMSPluginBase):
         # save() again on the Text instance (aka obj in this context) to update mptt values (numchild, etc).
         # See this ticket for details https://github.com/divio/djangocms-text-ckeditor/issues/212
         obj.clean_plugins()
+        obj.copy_referenced_plugins()
 
     def get_action_token(self, request, obj):
-        plugin_id = force_text(obj.pk)
+        plugin_id = force_str(obj.pk)
         # salt is different for every user
         signer = signing.Signer(salt=request.session.session_key)
         return signer.sign(plugin_id)
