@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 import json
 from copy import deepcopy
 
@@ -16,14 +15,14 @@ from . import settings as text_settings
 
 
 # this path is changed automatically whenever you run `gulp bundle`
-PATH_TO_JS = 'djangocms_text_ckeditor/js/dist/bundle-21269dba47.cms.ckeditor.min.js'
+PATH_TO_JS = 'djangocms_text_ckeditor/js/dist/bundle-bb31a8b837.cms.ckeditor.min.js'
 
 
 class TextEditorWidget(forms.Textarea):
     def __init__(self, attrs=None, installed_plugins=None, pk=None,
                  placeholder=None, plugin_language=None, configuration=None,
                  cancel_url=None, render_plugin_url=None, action_token=None,
-                 delete_on_cancel=False):
+                 delete_on_cancel=False, body_css_classes=''):
         """
         Create a widget for editing text + plugins.
 
@@ -36,12 +35,12 @@ class TextEditorWidget(forms.Textarea):
         if self.ckeditor_class not in attrs.get('class', '').join(' '):
             new_class = attrs.get('class', '') + ' %s' % self.ckeditor_class
             attrs.update({
-                'class': new_class.strip()
+                'class': new_class.strip(),
             })
         attrs.update({
-            'data-ckeditor-basepath': text_settings.TEXT_CKEDITOR_BASE_PATH
+            'data-ckeditor-basepath': text_settings.TEXT_CKEDITOR_BASE_PATH,
         })
-        super(TextEditorWidget, self).__init__(attrs)
+        super().__init__(attrs)
         self.installed_plugins = installed_plugins
         self.pk = pk
         self.placeholder = placeholder
@@ -56,21 +55,22 @@ class TextEditorWidget(forms.Textarea):
         self.render_plugin_url = render_plugin_url
         self.action_token = action_token
         self.delete_on_cancel = delete_on_cancel
+        self.body_css_classes = body_css_classes
 
     @property
     def media(self):
         return forms.Media(
             css={
-                'all': ('djangocms_text_ckeditor/css/cms.ckeditor.css',)
+                'all': ('djangocms_text_ckeditor/css/cms.ckeditor.css',),
             },
             js=(
                 static_with_version('cms/js/dist/bundle.admin.base.min.js'),
                 static(PATH_TO_JS),
-            )
+            ),
         )
 
     def render_textarea(self, name, value, attrs=None, renderer=None):
-        return super(TextEditorWidget, self).render(name, value, attrs, renderer)
+        return super().render(name, value, attrs, renderer)
 
     def render_additions(self, name, value, attrs=None, renderer=None):
         # id attribute is always present when rendering a widget
@@ -89,6 +89,8 @@ class TextEditorWidget(forms.Textarea):
         else:
             configuration['toolbar'] = configuration.get('toolbar', 'HTMLField')
 
+        configuration['bodyClass'] = self.body_css_classes
+
         config = json.dumps(configuration, cls=DjangoJSONEncoder)
         context = {
             'ckeditor_class': self.ckeditor_class,
